

// This Arduino Code corresponds to the SAP Watering Pot design by Charlotte Lee (cyl53)
// The loop () portion of the code has 4 parts: sensors (code from Arduino IDE), motor, LED, delay
//    1. SENSORS: tells ultrasonic sensors to detect distances of water and plant
//    2. MOTOR: controls rotation of micro servo motor, based on input data from sensor "Sensing Plant Pot"
//    3. LED: controls LED, which lights up based on input data from sensor "Sensing Water Level"
//    4. DELAY: delays rotation of motor and sensing of sensors until next time plant needs to be watered

#include <Servo.h>

Servo myservo;

const int water_sensor_pin_trig = 13;
const int water_sensor_pin_echo = 12;
const int plant_sensor_pin_trig = 11;
const int plant_sensor_pin_echo = 10;
const int servo_pin = 9;
const int LED_pin = 8;

bool within_5cm = false; // determines if spout is too close to sensor
int angle = 0;  // declare variable for servo angle
int dt = 15;    // short delay time 
int increment = 2; // amount that angle of servo motor changes by
int time_between_watering = 10000; // amount of time between when plant needs to be watered
long h = 5; // level that water in watering pot needs to be at or above


void setup()
{
  myservo.attach(servo_pin);
  pinMode(LED_pin, OUTPUT);
  pinMode(water_sensor_pin_trig, OUTPUT);
  pinMode(water_sensor_pin_echo, INPUT);
  pinMode(plant_sensor_pin_trig, OUTPUT);
  pinMode(plant_sensor_pin_echo, INPUT);
  Serial.begin(9600);
}

void loop()
{
  // 1. SENSORS
  
  // The following code chunk was copied and then modified from the "Sensors - Ping" example code in the Arduino IDE
  // --------------------------------------------------------------------------------------------------------------------------- //
  
  // FOR WATER LEVEL SENSOR 
  long duration_water, cm_water; // initialize variables for measuring water level

  // The PING))) is triggered by a HIGH pulse of 2 or more microseconds.
  // Give a short LOW pulse beforehand to ensure a clean HIGH pulse:
 
  digitalWrite(water_sensor_pin_trig, LOW);
  delayMicroseconds(2);
  digitalWrite(water_sensor_pin_trig, HIGH);
  delayMicroseconds(5);
  digitalWrite(water_sensor_pin_trig, LOW);

  duration_water = pulseIn(water_sensor_pin_echo, HIGH); //time (ms) from sending of ping to reception of its echo off of object

  cm_water = microsecondsToCentimeters(duration_water); // convert the time into a distance

  delay(100);

  Serial.print("Distance: ");
  Serial.print(cm_water);
  Serial.println(" cm");
  
  // PLANT DISTANCE SENSOR
  long duration_plant, cm_plant; // initialize variables for measuring plant distance

  // The PING))) is triggered by a HIGH pulse of 2 or more microseconds.
  // Give a short LOW pulse beforehand to ensure a clean HIGH pulse:

  digitalWrite(plant_sensor_pin_trig, LOW);
  delayMicroseconds(2);
  digitalWrite(plant_sensor_pin_trig, HIGH);
  delayMicroseconds(5);
  digitalWrite(plant_sensor_pin_trig, LOW);

  duration_plant = pulseIn(plant_sensor_pin_echo, HIGH); //time (ms) from sending of ping to reception of its echo off of object

  cm_plant = microsecondsToCentimeters(duration_plant); // convert the time into a distance
  delay(100); 
  // --------------------------------------------------------------------------------------------------------------------------- //

  
  
  
  // 2. MOTOR

  bool watered = false; // declare variable that tells whether plant has been watered yet
  if (angle<180 && within_5cm == false) // rotate forward if spout is far enough from plant
  { 
    angle = angle + increment; // increment angle
    myservo.write(angle); // move to new angle
    delay(dt);  
    if (cm_plant < 5) // if spout is too close to plant, change boolean within_5cm to stop from rotating more
    {
      within_5cm = true;
    }
  }
  else // rotate back to rest position (angle = 0)
  {
    if (angle >= 0) // if not at rest position yet
    {
        angle = angle - increment; // increment angle
        myservo.write(angle); // move to new angle
        delay(dt); 
    }
    else // once pot has returned to rest position
    {
      within_5cm = false;
      watered = true;
    }
  }
  
  
  
  
  // 3. LED
  if (cm_water > h) // Turn on LED if the level of the water in the pot is low
  {
    digitalWrite(LED_pin, HIGH); // LED on
  }
  else
  {
    digitalWrite(LED_pin, LOW); // LED off
  }
  
  
  
  // 4. DELAY
  if (watered == true) // delays cycle from repeating by time "time_between_watering"
  {
    delay(time_between_watering);
  }
}



// The following code chunk was copied and then modified from the "Sensors - Ping" example code in the Arduino IDE
//
// ---------------------------------------------------------------------------------------------------------------------- //

long microsecondsToCentimeters(long microseconds) {
  // The speed of sound is 340 m/s or 29 microseconds per centimeter.
  // The ping travels out and back, so to find the distance of the object we
  // take half of the distance travelled.
  return microseconds / 29 / 2;
}
//
// ---------------------------------------------------------------------------------------------------------------------- //
